"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processLogBuilder = exports.convertLevel = void 0;
const p_retry_1 = __importDefault(require("p-retry"));
const exit_hook_1 = __importDefault(require("exit-hook"));
// Define log sending limits
// https://docs.datadoghq.com/api/latest/logs/#send-logs
// Use 4.9 MB instead of 5 MB to determine when to stop batching
const LOGS_PAYLOAD_SIZE_LIMIT = 5138022;
// Use 0.95 MB instead of 1 MB to account for tags and other metadata
const LOG_SIZE_LIMIT = 996147;
const FORCE_SEND_MS = 3000;
const MAX_LOG_ITEMS = 995;
const convertLevel = (level) => {
    if (typeof level === 'string') {
        return level;
    }
    if (level == 60) {
        return 'fatal';
    }
    if (level >= 50) {
        return 'error';
    }
    if (level >= 40) {
        return 'warning';
    }
    if (level >= 30) {
        return 'log';
    }
    if (level >= 20) {
        return 'info';
    }
    return 'debug';
};
exports.convertLevel = convertLevel;
function processLogBuilder(options, apiInstance) {
    let logItemsLength = 0;
    let logItems = [];
    let timer = null;
    function sendLogs(logsToSend) {
        logItems = [];
        logItemsLength = 0;
        (0, p_retry_1.default)(async () => {
            if (options.onDebug) {
                options.onDebug(`Sending ${logsToSend.length} logs to datadog`);
            }
            const params = {
                body: logsToSend,
                contentEncoding: 'gzip',
            };
            try {
                const result = await apiInstance.submitLog(params);
                if (options.onDebug) {
                    options.onDebug(`Sending ${logsToSend.length} logs to datadog completed`);
                }
                return result;
            }
            catch (err) {
                throw {
                    err,
                    logs: logsToSend,
                };
            }
        }, { retries: options.retries ?? 5 }).catch(({ err, logs }) => {
            if (options.onError) {
                options.onError(err, logs);
            }
        });
    }
    if (!options.sendImmediate) {
        timer = setInterval(() => {
            if (logItems.length > 0) {
                // ...logItems is so if we clear logItems in another run, we won't lose these logs
                sendLogs([...logItems]);
            }
        }, options.sendIntervalMs || FORCE_SEND_MS);
    }
    (0, exit_hook_1.default)(() => {
        if (logItems.length > 0) {
            // Attempt to send logs on an exit call
            if (options.onDebug) {
                options.onDebug(`Shutdown detected. Attempting to send remaining logs to Datadog`);
            }
            if (timer) {
                clearInterval(timer);
            }
            sendLogs([...logItems]);
        }
    });
    return async function processLogs(source) {
        for await (const obj of source) {
            if (!obj) {
                return;
            }
            const logItem = {
                message: JSON.stringify({
                    ...obj,
                    level: (0, exports.convertLevel)(obj.level),
                }),
            };
            if (options.ddsource) {
                logItem.ddsource = options.ddsource;
            }
            if (options.ddtags) {
                logItem.ddtags = options.ddtags;
            }
            if (options.service) {
                logItem.service = options.service;
            }
            if (obj.hostname) {
                logItem.hostname = obj.hostname;
            }
            const logEntryLength = logItem.message.length +
                (logItem?.ddsource?.length || 0) +
                (logItem?.ddtags?.length || 0) +
                (logItem?.hostname?.length || 0) +
                (logItem?.service?.length || 0);
            if (logEntryLength > LOG_SIZE_LIMIT) {
                if (options.onError) {
                    options.onError(new Error(`Log entry exceeds size limit of ${LOG_SIZE_LIMIT} bytes: ${logEntryLength}`), [
                        logItem,
                    ]);
                }
            }
            logItems.push(logItem);
            logItemsLength += logEntryLength;
            const shouldSend = options.sendImmediate || logItemsLength > LOGS_PAYLOAD_SIZE_LIMIT || logItems.length > MAX_LOG_ITEMS;
            if (shouldSend) {
                sendLogs([...logItems]);
            }
        }
    };
}
exports.processLogBuilder = processLogBuilder;
//# sourceMappingURL=process-logs.js.map